Attribute VB_Name = "basTreeView"
Option Explicit

Public Const IIL_UNCHECKED As Long = 1
Public Const IIL_CHECKED As Long = 2

Public Const GWL_STYLE = (-16)

Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" ( _
    ByVal hwnd As Long, _
    ByVal nIndex As Long _
) As Long
     
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" ( _
    ByVal hwnd As Long, _
    ByVal nIndex As Long, _
    ByVal dwNewLong As Long _
) As Long

Declare Function SendMessageAny Lib "user32" Alias "SendMessageA" ( _
    ByVal hwnd As Long, _
    ByVal wMsg As Long, _
    ByVal wParam As Any, _
    lParam As Any _
) As Long

Public Type POINTAPI   'pt
  x As Long
  y As Long
End Type

Declare Function GetCursorPos Lib "user32" ( _
    lpPoint As POINTAPI _
) As Long
     
Declare Function ScreenToClient Lib "user32" ( _
    ByVal hwnd As Long, _
    lpPoint As POINTAPI _
) As Long
     
Declare Function GetAsyncKeyState Lib "user32" ( _
    ByVal vKey As Long _
) As Integer

Public Const TVS_CHECKBOXES  As Long = &H100  '>= IE3

Public Const TV_FIRST      As Long = &H1100
Public Const TVM_GETITEM   As Long = (TV_FIRST + 12)
Public Const TVM_SETITEM   As Long = (TV_FIRST + 13)
Public Const TVM_HITTEST   As Long = (TV_FIRST + 17)

Private Type TVITEM
    mask As Long
    hItem As Long
    State As Long
    stateMask As Long
    Text As String
    TextMax As Long
    iImage As Long
    iSelectedImage As Long
    Children As Long
    lParam As Long
End Type

'TVITEM.mask flags
Public Const TVIF_TEXT     As Long = &H1
Public Const TVIF_STATE    As Long = &H8
Public Const TVIF_HANDLE   As Long = &H10

'TVITEM.state bit value
Public Const TVIS_STATEIMAGEMASK  As Long = &HF000

Public Type TVHITTESTINFO   'was TV_HITTESTINFO
  pt As POINTAPI
  flags As Long
  hItem As Long
End Type

'TVHITTESTINFO.flags value
Public Const TVHT_ONITEMSTATEICON  As Long = &H40

Public Const MAX_ITEM = 256
Public Const TVM_GETNEXTITEM  As Long = (TV_FIRST + 10)

'TVM_GETNEXTITEM wParam values
Public Enum TVGN_Flags
    TVGN_ROOT = &H0
    TVGN_NEXT = &H1
    TVGN_PREVIOUS = &H2
    TVGN_PARENT = &H3
    TVGN_CHILD = &H4
    TVGN_FIRSTVISIBLE = &H5
    TVGN_NEXTVISIBLE = &H6
    TVGN_PREVIOUSVISIBLE = &H7
    TVGN_DROPHILITE = &H8
    TVGN_CARET = &H9
#If (WIN32_IE >= &H400) Then
    TVGN_LASTVISIBLE = &HA
#End If
End Enum

Public Function TreeViewGetItemText(hwndTV As Long, hItem As Long) As String
    Dim item As TVITEM
    Dim strText As String * 256
    Dim nLen As Long
    Dim nRes As Long
    
    item.hItem = hItem
    item.mask = TVIF_TEXT
    item.Text = Space(256)
    item.TextMax = 255
    Call SendMessageAny(hwndTV, TVM_GETITEM, 0&, item)
    
    TreeViewGetItemText = Left(item.Text, InStr(item.Text, Chr(0)) - 1)
End Function

Public Function IsTVItemChecked(hwndTV As Long, hItem As Long) As Boolean
   Dim tvi As TVITEM

   With tvi
      .mask = TVIF_STATE
      .hItem = hItem
      .stateMask = TVIS_STATEIMAGEMASK
   End With
   
   Call TreeView_GetItem(hwndTV, tvi)
   
   IsTVItemChecked = (tvi.State And INDEXTOSTATEIMAGEMASK(IIL_CHECKED))
End Function

Public Function IsTVItemCheckedFromClick(hwnd As Long, x As Long, y As Long) As Boolean
   Dim tvhTI As TVHITTESTINFO
   Dim fChecked As Boolean
    
   tvhTI.pt.x = x
   tvhTI.pt.y = y
  
   If TreeView_HitTest(hwnd, tvhTI) Then
      fChecked = IsTVItemChecked(hwnd, tvhTI.hItem)
      If (tvhTI.flags And TVHT_ONITEMSTATEICON) Then fChecked = Not fChecked
      IsTVItemCheckedFromClick = fChecked
   End If
End Function

Public Function INDEXTOSTATEIMAGEMASK(nState As Long) As Long
    INDEXTOSTATEIMAGEMASK = nState * (2 ^ 12)
End Function

Public Function TreeView_HitTest(hwnd As Long, lpHT As TVHITTESTINFO) As Long
   TreeView_HitTest = SendMessageAny(hwnd, TVM_HITTEST, 0&, lpHT)
End Function

Public Function TreeView_GetItem(hwnd As Long, pItem As TVITEM) As Boolean
    TreeView_GetItem = SendMessageAny(hwnd, TVM_GETITEM, 0&, pItem)
End Function

Public Function TreeView_GetNextItem(hwnd As Long, hItem As Long, code As Long) As Long
    TreeView_GetNextItem = SendMessageAny(hwnd, TVM_GETNEXTITEM, ByVal code, ByVal hItem)
End Function

Public Function TreeView_GetRoot(hwnd As Long) As Long
    TreeView_GetRoot = TreeView_GetNextItem(hwnd, 0, TVGN_ROOT)
End Function

Public Function TreeView_GetChild(hwnd As Long, hItem As Long) As Long
    TreeView_GetChild = TreeView_GetNextItem(hwnd, hItem, TVGN_CHILD)
End Function
    
Public Function TreeView_GetNextSibling(hwnd As Long, hItem As Long) As Long
    TreeView_GetNextSibling = TreeView_GetNextItem(hwnd, hItem, TVGN_NEXT)
End Function

Public Function TreeViewGetCurrentItem(hwnd As Long) As Long
    TreeViewGetCurrentItem = SendMessageAny(hwnd, TVM_GETNEXTITEM, TVGN_CARET, ByVal 0)
End Function

Public Sub TreeViewTrace(hwndTV As Long)
    Dim hItem As Long
   
    hItem = TreeView_GetRoot(hwndTV)
    
    Call PrintItem(hwndTV, hItem, 0)
    
    Call TreeViewTraceChildren(hwndTV, hItem, 1)
    
End Sub

Public Sub TreeViewTraceChildren(hwndTV As Long, hItem As Long, nLevel As Long)
    Dim hItemSibling As Long
    
    hItemSibling = TreeView_GetChild(hwndTV, hItem)
    
    While hItemSibling <> 0
        Call PrintItem(hwndTV, hItemSibling, nLevel)
        Call TreeViewTraceChildren(hwndTV, hItemSibling, nLevel + 1)
        hItemSibling = TreeView_GetNextSibling(hwndTV, hItemSibling)
        
    Wend
End Sub

Public Sub PrintItem(hwndTV As Long, hItem As Long, nLevel As Long)
    Debug.Print Space(nLevel) & TreeViewGetItemText(hwndTV, hItem) & "(" & IsTVItemChecked(hwndTV, hItem) & ")" & Hex(hItem)
End Sub

